@@ -9,6 +9,7 @@ from .utils import user_agent, sanitize_secrets
 from .read_configs import read_configs
 from .api_v4 import api_v4
 from .api_extras import api_extras
+from .api_decode_from_web import api_decode_from_web
 from .exceptions import CloudFlareError, CloudFlareAPIError, CloudFlareInternalError
 
 BASE_URL = 'https://api.cloudflare.com/client/v4'
@@ -154,6 +155,52 @@ class CloudFlare(object):
                               identifier1, identifier2, identifier3,
                               params, data, files)
 
+        def _connection(self, method, url, headers=None, params=None, data=None, files=None):
+            """ Cloudflare v4 API"""
+
+            if self.use_sessions:
+                if self.session is None:
+                    self.session = requests.Session()
+            else:
+                self.session = requests
+
+            method = method.upper()
+
+            if method == 'GET':
+                return self.session.get(url,
+                                        headers=headers, params=params, data=data)
+            if method == 'POST':
+                if isinstance(data, str):
+                    return self.session.post(url,
+                                             headers=headers, params=params, data=data, files=files)
+                else:
+                    return self.session.post(url,
+                                             headers=headers, params=params, json=data, files=files)
+            if method == 'PUT':
+                if isinstance(data, str):
+                    return self.session.put(url,
+                                            headers=headers, params=params, data=data)
+                else:
+                    return self.session.put(url,
+                                            headers=headers, params=params, json=data)
+            if method == 'DELETE':
+                if isinstance(data, str):
+                    return self.session.delete(url,
+                                               headers=headers, params=params, data=data)
+                else:
+                    return self.session.delete(url,
+                                               headers=headers, params=params, json=data)
+            if method == 'PATCH':
+                if isinstance(data, str):
+                    return self.session.request('PATCH', url,
+                                                headers=headers, params=params, data=data)
+                else:
+                    return self.session.request('PATCH', url,
+                                                headers=headers, params=params, json=data)
+
+            # should never happen
+            raise CloudFlareAPIError(0, 'method not supported')
+
         def _network(self, method, headers, parts,
                      identifier1=None, identifier2=None, identifier3=None,
                      params=None, data=None, files=None):
@@ -208,72 +255,10 @@ class CloudFlare(object):
                 self.logger.debug('Call: method and url %s %s', str(method), str(url))
                 self.logger.debug('Call: headers %s', str(sanitize_secrets(headers)))
 
-            method = method.upper()
-
-            if self.logger:
-                self.logger.debug('Call: doit!')
-
-            if self.use_sessions:
-                if self.session is None:
-                    self.session = requests.Session()
-            else:
-                self.session = requests
-
             try:
-                if method == 'GET':
-                    response = self.session.get(url,
-                                                headers=headers,
-                                                params=params,
-                                                data=data)
-                elif method == 'POST':
-                    if isinstance(data, str):
-                        response = self.session.post(url,
-                                                     headers=headers,
-                                                     params=params,
-                                                     data=data,
-                                                     files=files)
-                    else:
-                        response = self.session.post(url,
-                                                     headers=headers,
-                                                     params=params,
-                                                     json=data,
-                                                     files=files)
-                elif method == 'PUT':
-                    if isinstance(data, str):
-                        response = self.session.put(url,
-                                                    headers=headers,
-                                                    params=params,
-                                                    data=data)
-                    else:
-                        response = self.session.put(url,
-                                                    headers=headers,
-                                                    params=params,
-                                                    json=data)
-                elif method == 'DELETE':
-                    if isinstance(data, str):
-                        response = self.session.delete(url,
-                                                       headers=headers,
-                                                       params=params,
-                                                       data=data)
-                    else:
-                        response = self.session.delete(url,
-                                                       headers=headers,
-                                                       params=params,
-                                                       json=data)
-                elif method == 'PATCH':
-                    if isinstance(data, str):
-                        response = self.session.request('PATCH', url,
-                                                        headers=headers,
-                                                        params=params,
-                                                        data=data)
-                    else:
-                        response = self.session.request('PATCH', url,
-                                                        headers=headers,
-                                                        params=params,
-                                                        json=data)
-                else:
-                    # should never happen
-                    raise CloudFlareAPIError(0, 'method not supported')
+                if self.logger:
+                    self.logger.debug('Call: doit!')
+                response = self._connection(method, url, headers, params, data, files)
                 if self.logger:
                     self.logger.debug('Call: done!')
             except Exception as e:
@@ -296,7 +281,7 @@ class CloudFlare(object):
                 response_type = 'application/octet-stream'
             response_code = response.status_code
             response_data = response.content
-            if not isinstance(response_data, str):
+            if not isinstance(response_data, (str, bytes, bytearray)):
                 response_data = response_data.decode("utf-8")
 
             if self.logger:
@@ -364,6 +349,9 @@ class CloudFlare(object):
                     response_data = response_data.decode('utf-8')
                 try:
                     response_data = json.loads(response_data)
+                    if not isinstance(response_data, (dict)):
+                        response_data = {'success': True,
+                                         'result': response_data}
                 except ValueError:
                     if response_data == '':
                         # This should really be 'null' but it isn't. Even then, it's wrong!
@@ -397,23 +385,64 @@ class CloudFlare(object):
                     # 3xx & 4xx errors - we should report that somehow - but not quite yet
                     # response_data['code'] = response_code
                     pass
+            elif response_type == 'application/octet-stream' and isinstance(response_data, (int, float)):
+                # It's binary data
+                if response_code == requests.codes.ok:
+                    # 200 ok
+                    response_data = {'success': True,
+                                     'result': response_data}
+                else:
+                    # 3xx & 4xx errors
+                    response_data = {'success': False,
+                                     'code': response_code,
+                                     'result': response_data}
+            elif response_type == 'application/octet-stream' and isinstance(response_data, (bytes, bytearray)):
+                # API says it's text; but maybe it's actually JSON? - should be fixed in API
+                if hasattr(response_data, 'decode'):
+                    response_data = response_data.decode('utf-8')
+                try:
+                    response_data = json.loads(response_data)
+                    if not isinstance(response_data, (dict)) or 'success' not in response_data:
+                        if response_code == requests.codes.ok:
+                            # 200 ok
+                            response_data = {'success': True,
+                                             'result': response_data}
+                        else:
+                            # 3xx & 4xx errors
+                            response_data = {'success': False,
+                                             'code': response_code,
+                                             'result': response_data}
+                except ValueError:
+                    # So it wasn't JSON - moving on as if it's text!
+                    # A single value is returned (vs an array or object)
+                    if response_code == requests.codes.ok:
+                        # 200 ok
+                        response_data = {'success': True, 'result': response_data}
+                    else:
+                        # 3xx & 4xx errors
+                        response_data = {'success': False,
+                                         'code': response_code,
+                                         'result': response_data}
             elif response_type == 'text/plain' or response_type == 'application/octet-stream':
                 # API says it's text; but maybe it's actually JSON? - should be fixed in API
                 if hasattr(response_data, 'decode'):
                     response_data = response_data.decode('utf-8')
                 try:
                     response_data = json.loads(response_data)
+                    if not isinstance(response_data, (dict)):
+                        response_data = {'success': True,
+                                         'result': response_data}
                 except ValueError:
                     # So it wasn't JSON - moving on as if it's text!
                     # A single value is returned (vs an array or object)
                     if response_code == requests.codes.ok:
                         # 200 ok
-                        response_data = {'success': True, 'result': str(response_data)}
+                        response_data = {'success': True, 'result': response_data}
                     else:
                         # 3xx & 4xx errors
                         response_data = {'success': False,
                                          'code': response_code,
-                                         'result': str(response_data)}
+                                         'result': response_data}
             elif response_type == 'text/javascript' or response_type == 'application/javascript':
                 # used by Cloudflare workers
                 if response_code == requests.codes.ok:
@@ -566,6 +595,25 @@ class CloudFlare(object):
             result = response_data
             return result
 
+        def _api_from_web(self):
+            """ Cloudflare v4 API"""
+
+            # base url isn't enough; we need less
+            url = '/'.join(self.base_url.split('/')[0:3])
+
+            try:
+                if self.logger:
+                    self.logger.debug('Call: doit!')
+                response = self._connection("GET", url)
+                if self.logger:
+                    self.logger.debug('Call: done!')
+            except Exception as e:
+                if self.logger:
+                    self.logger.debug('Call: exception! "%s"' % (e))
+                raise CloudFlareAPIError(0, 'connection failed.')
+
+            return response.text
+
     class _AddUnused(object):
         """ Cloudflare v4 API"""
 
@@ -890,6 +938,11 @@ class CloudFlare(object):
                 w = w + self.api_list(a, s + '/' + n)
         return w
 
+    def api_from_web(self):
+        """ Cloudflare v4 API"""
+
+        return api_decode_from_web(self._base._api_from_web())
+
     def __init__(self, email=None, token=None, certtoken=None, debug=False, raw=False, use_sessions=True, profile=None):
         """ Cloudflare v4 API"""
 
